#!/usr/bin/perl -w
#
# $Id: gen-unihan-strings.pl,v 1.1 2003/12/30 19:33:31 nlevitt Exp $ 
#
# see usage notes below
#

use strict;
use vars ('$UNZIP', '$UNICODE_VERSION');

sub write_unihan_h ($$);

# if these things aren't in your path you can put full paths to them here
$UNZIP = 'unzip';

sub process_unihan_zip ($);

$| = 1;  # flush stdout buffer

if (@ARGV != 2) 
{
    $0 =~ s@.*/@@;
    die <<EOF
Usage: $0 UNICODE-VERSION Unihan.zip
EOF
}

$UNICODE_VERSION = $ARGV[0];
my $unihan_zip = $ARGV[1];

open (my $unihan, "-|:crlf:utf8", "$UNZIP -c $unihan_zip") or die;
open (my $out, ">:utf8", "unicode-unihan.h") or die;

write_unihan_h ($unihan, $out);

close ($unihan);
close ($out);

exit;


#------------------------#

sub write_unihan_h ($$)
{
    my ($unihan, $out) = @_;

    my ($definition, $frequency);
    my $wc = "0000";

    my %frequencies = ( 1 => 'VERY COMMON',
                        2 => 'SOMEWHAT COMMON',
                        3 => 'NOT UNCOMMON',
                        4 => 'SOMEWHAT UNCOMMON',
                        5 => 'VERY UNCOMMON',
                        6 => 'FREQUENCY UNSPECIFIED' );
    my %characters; # hash of hashes by frequency

    print "please wait";

    print $out qq|/* unicode-unihan.h */\n|;
    print $out qq|/* THIS IS A GENERATED FILE. CHANGES WILL BE OVERWRITTEN. */\n|;
    print $out qq|/* This file is part of the unicode-han-translation project. */\n|;
    print $out qq|/* Generated by $0 */\n|;
    print $out qq|\n|;
    print $out qq|/*\n|;
    print $out qq| * This file isn't meant to be used as real C code. It is for gettext to\n|;
    print $out qq| * parse and create the pot file. I'm doing it this way so that gettext can\n|;
    print $out qq| * work its magic with fuzzy strings and whatnot. This could change in the\n|;
    print $out qq| * future.\n|;
    print $out qq| */\n|;
    print $out qq|\n|;
    print $out qq|/* Prevent the above comments from showing up in the pot file */\n|;
    print $out qq|;\n|;
    print $out qq|\n|;
    print $out qq|/* Unicode version $UNICODE_VERSION. */\n|;
    print $out qq|/* */\n|;
    print $out qq|/* Characters are ordered by frequency according to kFrequency in */\n|;
    print $out qq|/* Unihan.zip, converted to English like so: */\n|;
    print $out qq|/* */\n|;
    print $out qq|/*   1 => $frequencies{1} */\n|;
    print $out qq|/*   2 => $frequencies{2} */\n|;
    print $out qq|/*   3 => $frequencies{3} */\n|;
    print $out qq|/*   4 => $frequencies{4} */\n|;
    print $out qq|/*   5 => $frequencies{5} */\n|;
    print $out qq|/* */\n\n|;

    my $i = 0;
    while (my $line = <$unihan>)
    {
        chomp $line;
        $line =~ /^U\+([0-9A-F]+)\s+([^\s]+)\s+(.+)$/ or next;

        my $new_wc = $1;
        my $field = $2;
        my $value = $3;

        if ($field eq 'kFrequency')
        {
            $frequency = int ($value);
        }
        elsif ($field eq 'kDefinition')
        {
            $definition = $value;
            $definition =~ s/\"/\\"/g;
        }

        if ($new_wc ne $wc)
        {
            if ($definition)
            {
                if ($frequency)
                {
                    $characters{$frequency}{hex ($wc)} = $definition;
                }
                else
                {
                    $characters{6}{hex ($wc)} = $definition;
                }
            }

            undef $frequency;
            undef $definition;
            $wc = $new_wc;
        }

        if ($i++ % 32768 == 0) 
        {
            print ".";
        }
    }

    for $frequency (sort keys %characters)
    {
        for $wc (sort {$a <=> $b} keys %{$characters{$frequency}})
        {
            my $character = chr ($wc);
            $definition = $characters{$frequency}{$wc};

            printf $out (qq|/* U+%04X \%s [\%s] */\n|, $wc, $character, $frequencies{$frequency});
            print $out qq|N_("$definition");\n\n|;
        }
    }

    print " done.\n";
}
